<?php
namespace FiloBlu\ProductUrlTools\Console\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Magento\UrlRewrite\Model\UrlPersistInterface;
use Magento\CatalogUrlRewrite\Model\ProductUrlRewriteGenerator;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Store\Model\Store;
use Magento\Framework\App\State;

class RegenerateProductUrlKeyCommand extends Command
{
    /**
     * @var ProductUrlRewriteGenerator
     */
    protected $productUrlRewriteGenerator;

    /**
     * @var UrlPersistInterface
     */
    protected $urlPersist;

    /**
     * @var ProductRepositoryInterface
     */
    protected $collection;

    /**
     * @var \Magento\Framework\App\State
     */
    protected $state;
    protected $logger;
    protected $action;
    protected $_resource;


    public function __construct(
        State $state,
        Collection $collection,
        ProductUrlRewriteGenerator $productUrlRewriteGenerator,
        UrlPersistInterface $urlPersist,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Catalog\Model\Product\Attribute\Source\Status $productStatus,
        \Magento\Catalog\Model\Product\Visibility $productVisibility,
        \Magento\Catalog\Model\ResourceModel\Product\Action $action,
        \Magento\Framework\App\ResourceConnection $resource

    ) {
        $this->state = $state;
        $this->collection = $collection;
        $this->productUrlRewriteGenerator = $productUrlRewriteGenerator;
        $this->urlPersist = $urlPersist;
        $this->logger = $logger;
        $this->productVisibility = $productVisibility;
        $this->productStatus = $productStatus;
        $this->action= $action;
        $this->_resource = $resource;


        parent::__construct();
    }

    protected function configure()
    {
        $this->setName('tools:regenerateurlkey')
            ->setDescription('Regenerate urlkey for given products')
            ->addArgument(
                'pids',
                InputArgument::IS_ARRAY ,
                'Products to regenerate'
            )
            ->addOption(
                'store', 's',
                InputOption::VALUE_REQUIRED,
                'Use the specific Store View'
            )
            ->addOption(
                'all_visible', 'a',
                InputOption::VALUE_OPTIONAL,
                'Use for all product visible'
            )
        ;
        return parent::configure();
    }

    public function execute(InputInterface $inp, OutputInterface $out)
    {
        $pids = $inp->getArgument('pids');
        $store_id = $inp->getOption('store');
        $all_visible=$inp->getOption('all_visible');
        
        $allStores = $this->getAllStoreIds();
        
        if( !empty($pids) )
            $this->collection->addIdFilter($pids);
        
        $this->getCollection($store_id);
        
        if($all_visible)
        {
            $this->all_productVisible();
        }


        if (!empty($store_id) ) {
            if (isset($allStores[$store_id]))
            {
                $storesList = array(
                    $store_id => $allStores[$store_id]);
            }
        }
        else {
            $storesList = $allStores;
        }


        foreach ($storesList as $store_id)
        {
            $this->collection->clear();
            $this->collection->setStore($store_id);

            $list = $this->collection->load();
            
            foreach ($list as $product) {

                //$this->logger->info($product->getName());
                if (!$product->getName()) {
                    $this->logger->info('name is empty for product : ' . $product->getId() . ' in store view ' . $store_id);
                    continue;
                }
                $name = $product->getName();

                $urlKey = $product->formatUrlKey( $name. '-' . $store_id . $product->getId());
                $pids = array($product->getId());
                $attributesData = array("url_key" => $urlKey);

                try {
                    $this->action->updateAttributes($pids, $attributesData, $store_id);
                    $this->logger->info('processed urlkey for product with id : ' . $product->getId() . " storeID " . $store_id);
                } catch (\Exception $e) {
                    $out->writeln('<error>Duplicated url for ' . $product->getId() . ' ' . $e->getMessage() . ' </error>');
                }
            }
        }
    }

    private function all_productVisible()
    {
        $this->collection->addAttributeToFilter('status', ['in' => $this->productStatus->getVisibleStatusIds()]);
        $this->collection->setVisibility($this->productVisibility->getVisibleInSiteIds());
    }

    public function getAllStoreIds() {
        $result = [];

        $sql = $this->_resource->getConnection()->select()
            ->from($this->_resource->getTableName('store'), array('store_id', 'code'))
            // ->where('`code` <> ?', 'admin')
            ->order('store_id', 'ASC');

        $queryResult = $this->_resource->getConnection()->fetchAll($sql);

        foreach ($queryResult as $row) {
            if (isset($row['store_id']) && (int)$row['store_id'] > 0) {
                $result[(int)$row['store_id']] = $row['store_id'];
            }
        }

        return $result;
    }
    public function getCollection($store_id)
    {
        $this->collection->addAttributeToSelect(['name', 'url_key']);
        $this->collection->addStoreFilter($store_id )->setStoreId($store_id );

    }
}

